"use server";

import { getServerSession } from "next-auth";
import { AliasEntry, approveAliasEntry, createAlias, createUserEntry, database, deleteAliasEntry, getAlias, getAllAliases, getUserAliases, setUserPassword } from "./db";
import { aliasesNeedApproval, isAdmin } from "./util";

export async function fetchAllUsers(): Promise<string[]> {
  return new Promise(async (resolve, reject) => {
    const session = await getServerSession();
    if (!isAdmin(session)) return reject("Unauthenticated");

    const db = database('credentials');
    db.all("SELECT key FROM passwords", (err, res: any) => {
      if (err) return reject(err);

      resolve(res.map((row: any) => row.key));
    });
  });
}

export async function changeOwnPassword(newPass: string) {
  if (newPass.length < 8) throw new Error("Invalid password");
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");
  await setUserPassword(session.user.email, newPass);
}

export async function fetchOwnAliases() {
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");
  return await getUserAliases(session.user.email);
}

export async function fetchAllAliases() {
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");
  if (!isAdmin(session)) throw new Error("Unauthorized");

  return await getAllAliases();
}

export async function aliasAvailable(email: string) {
  const session = await getServerSession();
  if (!session?.user) throw new Error("Unauthenticated");

  return new Promise<boolean>((resolve, reject) => {
    const db = database('aliases');
    db.get('SELECT id FROM aliases WHERE alias = ?', email.toLowerCase(), (err, res) => {
      db.close();
      if (err) return reject(err);
      if (res != undefined) return resolve(false);

      const authDb = database('credentials');
      authDb.get('SELECT key FROM passwords WHERE key = ?', email.toLowerCase(), (err, res) => {
        authDb.close();
        if (err) return reject(err);
        resolve(res == undefined);
      });
    });
  });
}

export async function createAliasSelf(alias: string): Promise<AliasEntry> {
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");

  const pending = aliasesNeedApproval(session);
  if (!await aliasAvailable(alias)) throw new Error("Alias unavailable");
  const id = await createAlias(session.user.email, alias.toLowerCase(), pending);

  return {
    id: id,
    address: session.user.email,
    alias: alias,
    pending: pending
  };
}

export async function deleteAlias(alias: string) {
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");

  if (!isAdmin(session) && (await getAlias(alias))?.address != session.user.email) {
    throw new Error("Unauthorized");
  }

  await deleteAliasEntry(alias);
}

export async function approveAlias(alias: string) {
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");
  if (!isAdmin(session)) throw new Error("Unauthorized");

  await approveAliasEntry(alias);
}

export async function createUser(email: string, password: string) {
  const session = await getServerSession();
  if (!session?.user?.email) throw new Error("Unauthenticated");
  if (!isAdmin(session)) throw new Error("Unauthorized");

  if (!await aliasAvailable(email.toLowerCase())) {
    throw new Error("Alias unavailable");
  }

  if (password.length < 8 || !email.includes("@")) {
    throw new Error("Validation failed");
  }

  await createUserEntry(email, password);
}
